
/**
 * @fileoverview
 * Module with functions for parsing ASD feeds and for serializing
 * objects to the same.
 * 
 */

function ASDParser()
{
    // There need ever only be one instance since there is no state! this makes
    // it a singleton.
    if (ASDParser.instance) {
        return ASDParser.instance;
    }
    else
    {
        ASDParser.instance = this;
    }

    /**
     * Parse an xml document object into javascript objects.
     */
    this.parseDocument = function(doc)
    {
        if (! doc || ! doc.documentElement ||
            ! doc.documentElement.nodeName=="services")
        {
            // something was wrong with the feed!
            return null;
        }

        return this._doParse(doc.documentElement);
    }

    /**
     * Parse a string containing an xml document into javascript objects.
     */
    this.parseString = function(str)
    {
        var parser = new DOMParser();
        var doc = parser.parseFromString(str, "text/xml");
        return this.parseDocument(doc);
    }

    this._doParse = function(node)
    {
        var meta = this._parseMeta(node);
        var services = this._parseServices(node);
        meta.services = services;
        return meta;
    }

    /**
     * Parse all of the bits that are not <service> elements
     */
    this._parseMeta = function(node)
    {
        var ret = {};
          ret.total = node.getAttribute("total");
          ret.length = node.getAttribute("length");
        return ret;
    }
    
    /**
     * Parse all <entry> elements.
     */
    this._parseServices = function(node)
    {
        var ret = {};
        for (var n=0, element; element=node.childNodes[n]; n++)
        {
            if (element.nodeName == "service")
            {
                this._parseService(element, ret);
            }
        }
        return ret;
    }

    this._parseService = function(node, list)
    {
        var ret = {};
        try{
          ret.username = node.getAttribute("username");
          ret.access = node.getAttribute("access");
          ret.name = node.getAttribute("name");
          ret.author = node.getAttribute("author");
          ret.server = node.getAttribute("server");
          ret.originurl = node.getAttribute("originurl");
          ret.device = node.getAttribute("device");
          ret.path = node.getAttribute("path");
          ret.onlinesince = node.getAttribute("onlinesince");
          ret.description = node.getAttribute("description");
          ret.onlinefor = node.getAttribute("onlinefor");
        } catch(e){}
        var serviceKey = ret.server + ":" + ret.path;
        
        if(!list[serviceKey])
        {
          list[serviceKey] = {}
        }
        for(key in ret)
        {
            list[serviceKey][key] = ret[key];
        }
    }
}
