/*
 * Copyright (c) 2009 Opera Software ASA, António Afonso (antonio.afonso@opera.com)
 */
(function() {
    var pictureType = [
        "32x32 pixels 'file icon' (PNG only)",
        "Other file icon",
        "Cover (front)",
        "Cover (back)",
        "Leaflet page",
        "Media (e.g. lable side of CD)",
        "Lead artist/lead performer/soloist",
        "Artist/performer",
        "Conductor",
        "Band/Orchestra",
        "Composer",
        "Lyricist/text writer",
        "Recording Location",
        "During recording",
        "During performance",
        "Movie/video screen capture",
        "A bright coloured fish",
        "Illustration",
        "Band/artist logotype",
        "Publisher/Studio logotype"
    ];
    
    function getTextEncoding( bite ) {
        var charset;
        switch( bite )
        {
            case 0x00:
                charset = 'iso-8859-1';
                break;
                
            case 0x01:
                charset = 'utf-16';
                break;
                
            case 0x02:
                charset = 'utf-16be';
                break;
                
            case 0x03:
                charset = 'utf-8';
                break;
        }
        
        return charset;
    }
    
    function getTime( duration )
    {
        var duration    = duration/1000,
            seconds     = Math.floor( duration ) % 60,
            minutes     = Math.floor( duration/60 ) % 60,
            hours       = Math.floor( duration/3600 );
            
        return {
            seconds : seconds,
            minutes : minutes,
            hours   : hours
        };
    }
    
    function formatTime( time )
    {
        var seconds = time.seconds < 10 ? '0'+time.seconds : time.seconds;
        var minutes = (time.hours > 0 && time.minutes < 10) ? '0'+time.minutes : time.minutes;
        
        return (time.hours>0?time.hours+':':'') + minutes + ':' + seconds;
    }
    
    function getUTF16StringSize( data, offset )
    {
        for( var ix = offset, twoBytes = 1; twoBytes != 0; ix++ );
        {
            twoBytes = data.getInteger16At(ix++);
        }
        
        return ix - offset - 1;
    }
    
    //function getStringSize( string, charset )
    //{
    //    switch( charset )
    //    {
    //        case 'utf-8':
    //        case 'utf-16':
    //    }
    //    
    //    // iso-8859-1 and anything else
    //    return string.length;
    //}
    
    ID3v2.readFrameData['APIC'] = function readPictureFrame(offset, length, data, flags, v) {
        v = v || '3';
        
        var start = offset;
        var charset = getTextEncoding( data.getByteAt(offset) );
        switch( v ) {
            case '2':
                var format = data.getStringAt(offset+1, 3 );
                offset += 4;
                break;
                
            case '3':
            case '4':
                var format = data.getStringAt(offset+1, length );
                offset += 1+format.length + 1; // \0 byte
                break;
        }
        var bite = data.getByteAt(offset, 1);
        var type = pictureType[bite];
        var desc = data.getStringAt(offset+1, length, charset);
        
        offset += 1+(charset=='utf-16'?getUTF16StringSize(data, offset+1)+2:desc.length+1);
        
        //PSO.pub( 'afonso', 'v: ' + v );
        //PSO.pub( 'afonso', 'charset: ' + charset + '\ntype: ' + type + '\ndesc: ' + desc);
        //var hex = '';
        //for( var i = 0; i < 20; i++ )
        //{
        //    hex += toHex(data.getByteAt(offset+i)) + ' ';
        //}
        //PSO.pub( 'afonso', 'hex: ' + hex );
        
        return {
            format : format,
            type : type,
            description : desc,
            data : data.getBytesAt(offset, (start+length) - offset)
        };
    };
    
    ID3v2.readFrameData['COMM'] = function readCommentsFrame(offset, length, data) {
        var start = offset;
        var charset = getTextEncoding( data.getByteAt(offset) );
        var language = data.getStringAt( offset+1, 3 );
        var shortdesc = data.getStringAt( offset+4, length-4, charset );
        
        offset += 4 + (charset=='utf-16'?getUTF16StringSize(data, offset+1)+2:shortdesc.length+1);
        var text = data.getStringAt( offset, (start+length) - offset );
        
        return {
            language : language,
            short_description : shortdesc,
            text : text
        };
    };
    
    ID3v2.readFrameData['COM'] = ID3v2.readFrameData['COMM'];
    
    ID3v2.readFrameData['PIC'] = function(offset, length, data, flags) {
        return ID3v2.readFrameData['APIC'](offset, length, data, flags, '2');
    };
    
    ID3v2.readFrameData['PCNT'] = function readCounterFrame(offset, length, data) {
        // FIXME: implement the rest of the spec
        return data.getInteger32At(offset);
    };
    
    ID3v2.readFrameData['CNT'] = ID3v2.readFrameData['PCNT'];
    
    ID3v2.readFrameData['T*'] = function readTextFrame(offset, length, data) {
        var charset = getTextEncoding( data.getByteAt(offset) );
        
        
        //PSO.pub( 'afonso', 'charset: ' + charset );
        //PSO.pub( 'afonso', 's: ' + data.getStringAt(offset+1, length-1, 'euc-jp') );
        //PSO.pub( 'afonso', 'hex: ' + hex );
        
        return data.getStringAt(offset+1, length-1, charset);
        //return data.getStringAt(offset+1, length-1, 'euc-cn');
    };
    
    ID3v2.readFrameData['TCON'] = function readGenreFrame(offset, length, data) {
        var text = ID3v2.readFrameData['T*'].apply( this, arguments );
        return text.replace(/^\(\d+\)/, '');
    };
    
    ID3v2.readFrameData['TCO'] = ID3v2.readFrameData['TCON'];
    
    //ID3v2.readFrameData['TLEN'] = function readLengthFrame(offset, length, data) {
    //    var text = ID3v2.readFrameData['T*'].apply( this, arguments );
    //    
    //    return {
    //        text : text,
    //        parsed : formatTime( getTime(parseInt(text)) )
    //    };
    //};
    
    ID3v2.readFrameData['USLT'] = function readLyricsFrame(offset, length, data) {
        var start = offset;
        var charset = getTextEncoding( data.getByteAt(offset) );
        var language = data.getStringAt( offset+1, 3 );
        var descriptor = data.getStringAt( offset+4, length-4, charset );
        
        offset += 4 + (charset=='utf-16'?getUTF16StringSize(data, offset+1)+2:descriptor.length+1);
        var lyrics = data.getStringAt( offset, (start+length) - offset, charset );
        
        return {
            language : language,
            descriptor : descriptor,
            lyrics : lyrics
        };
    };
    
    ID3v2.readFrameData['ULT'] = ID3v2.readFrameData['USLT'];
})();