/*
 * Copyright (c) 2009 Opera Software ASA, António Afonso (antonio.afonso@opera.com)
 */
function ReadFile(url, callback, range)
{
    if( !opera.io || !opera.io.filesystem ) {
        throw "File I/O API not found.";
    }
    
    var file = opera.io.filesystem.mountSystemDirectory('shared').resolve(url);
    
    if( !file.exists ) {
        throw "File " + url + " does not exist.";
    }
    
    range = range || [0];
    // consider size-start if start < 0
    var start = range[0] < 0 ? file.fileSize+range[0] : range[0];
    var length = range[1] || file.fileSize-start;
    var stream = file.open( file, opera.io.filemode.READ );
    
    var data = {
        getStringAt: function(offset, length, charset) {
            stream.position = start+offset;
            // FIXME: UTF-16 takes too long to read #CORE-24874
            if( charset && charset.toLowerCase() == 'utf-16' ) { return ""; }
            return stream.read(length, charset);
        },
        getByteAt: function(offset) {
            stream.position = start+offset;
            return stream.readBytes(1)[0];
        },
        getBytesAt: function(offset, length) {
            stream.position = start+offset;
            return stream.readBytes(length);
        },
        getInteger32At: function(offset) {
            stream.position = start+offset;
            var bytes = stream.readBytes(4);
            return bytes[3]
                 | (bytes[2] << 8)
                 | (bytes[1] << 16)
                 | (bytes[0] << 24);
        },
        getInteger24At: function(offset) {
            stream.position = start+offset;
            var bytes = stream.readBytes(3);
            return bytes[2]
                 | (bytes[1] << 8)
                 | (bytes[0] << 16);
        },
        getInteger16At: function(offset) {
            stream.position = start+offset;
            var bytes = stream.readBytes(2);
            return bytes[1]
                 | (bytes[0] << 8);
        },
        isBitSetAt: function(offset, bit) {
            var octect = this.getByteAt(offset);
            var mask = 1 << bit;
            return (octect & mask) != 0;
        },
        getLength: function() {
            return stream.bytesAvailable;
        },
        close: function() {
            return stream.close();
        }
    };
    
    if( callback ) { callback(data); }
    
    return data;
}
