Yusef.addPlugin
(
    'graphics',
    function Yusef_plugins_graphics()
    {
        var undefined;
        var that            = this;
        var _config         = {};
        _config.pluginName  = 'Yusef.plugins.graphics';
        _config.cacheMaxAge = 1000*60*60*24*30;         //  30 days max age without being accessed
        _config.busyMaxAge  = 1000*60;                  //  1 minute between the busy notifications
        _config.busyString  = 'Generating preview images. Your computer may slow down.';




        var _lastTimeBusy   = 0;
        var _cache          = {};


        //  --
        var savedInfo = widget.preferenceForKey( _config.pluginName+'_cache' )||'';
        if( savedInfo!=='' )
        {
            _cache = JSON.parse( savedInfo );
        }
        PSO.pub( _config.pluginName+'_cache', _enumerateObject( _cache, '_cache RESTORED' ) );




        /*
         *  _showBusyNotification
         */
        function _showBusyNotification()
        {
            var now = new Date().getTime();
            if( now-_config.busyMaxAge>_lastTimeBusy )
            {
                window.status = _(_config.busyString);
                setTimeout( function(){window.status=''}, _config.busyMaxAge/2 );
            }
            _lastTimeBusy   = now;
        }

        
        /*
         *  @private
         */
        function _getCachedFile( image, hash )
        {
            var fileObject  = Yusef.mountpoints.storage.resolve( hash );
            if( fileObject.exists==true && _cache[image.path]!==undefined && _cache[image.path].versions[hash]!==undefined && _cache[image.path].versions[hash]==image.modified.getTime() )
            {
                return fileObject;
            }
            return false;
        }


        /*
         *  @private
         */
        function _updateCache( image, hash )
        {
            if( !image || !image.exists )
            {
                return false;
            }
            
            var touched     = false;
            var modified    = image.modified.getTime();
            var accessed    = new Date().getTime();
            var path        = image.path;

            if( !_cache[path] )
            {
                _cache[path] = { 'versions':{} };
            }
            _cache[path].accessed       = accessed;
            _cache[path].modified       = modified;
            
            if( !_cache[path].versions[hash] ){ touched=true; }
            _cache[path].versions[hash] = modified;


            //  purge old _cache entries
            var accessedMin = accessed-_config.cacheMaxAge;
            for( var k in _cache )
            {
                if( _cache[k].accessed<accessedMin || _cache[k].accessed==undefined )
                {
                    //  TODO:   delete the files _cache[k].versions[ fileName ]
                    touched     = true;
                    delete _cache[k];
                }
            }


            //  save _cache
            if( touched )
            {
                widget.setPreferenceForKey( JSON.stringify( _cache ), _config.pluginName+'_cache' );
            }
        }


        /*
         *  resizeImage
         */
        this.resizeImage = function( image, args, callback )
        {
            //  clean up the args
            args.maxWidth       = args.maxWidth||64;
            args.maxHeight      = args.maxHeight||64;
            args.crop           = args.crop||false;
            args.quality        = args.quality||.1;
            args.angle          = (args.angle||0)&3;

            //  add the unHashed and hash resized image name
            args.unHashed       = ([image.path,args.maxWidth,args.maxHeight,args.crop,args.quality,args.angle]).join('\n');
            args.hash           = (args.hash||_hash( args.unHashed ))+'.jpg';
            
            //  try to get cached file and pass it to the callback directly.
            var cachedFile    = _getCachedFile( image, args.hash );
            if( cachedFile )
            {
                return callback( {'file':cachedFile} );
            }

            //  show busy notification
            _showBusyNotification();

            //  resizeImage_handler
            function resizeImage_handler( event )
            {
                var eventType   = event.type.toLowerCase();
                var srcImage    = event.srcElement;


                if( eventType!=='load' )
                {
                    callback();
                }
                else
                {
                    var tw      = srcImage.width;
                    var th      = srcImage.height;
                    var canvas  = document.createElement( 'canvas' );

                    if( args.crop )
                    {
                        //  crop resizing
                        var srcAR   = tw/th;
                        var dstAR   = args.maxWidth/args.maxHeight;

                        if( srcAR>dstAR && th>args.maxHeight )
                        {
                            tw *= args.maxHeight/th;
                            th  = args.maxHeight;
                        }
                        else if( srcAR<dstAR && tw>args.maxWidth )
                        {
                            th *= args.maxWidth/tw;
                            tw = args.maxWidth;
                        }

                        var w=args.maxWidth;
                        var h=args.maxHeight;
                    }
                    else
                    {
                        //  fit resizing
                        if( tw>args.maxWidth )
                        {
                            th *= args.maxWidth/tw;
                            tw = args.maxWidth;
                        }
                        if( th>args.maxHeight )
                        {
                            tw *= args.maxHeight/th;
                            th = args.maxHeight;
                        }

                        var w=tw;
                        var h=th;
                    }

                    //  rotation on one side ?
                    if( args.angle&1 )
                    {
                        var tmp = w;
                        w = h;
                        h = tmp;
                    }

                    canvas.width    = Math.round(w)&-2;
                    canvas.height   = Math.round(h)&-2;
                    var context     = canvas.getContext( '2d' );

                    context.fillStyle   = '#fff';
                    context.fillRect( 0,0, canvas.width, canvas.height );

                    context.save();
                    context.translate( canvas.width/2, canvas.height/2 );
                    context.rotate( Math.PI/2*args.angle );
                    context.drawImage( srcImage,0,0, srcImage.width, srcImage.height, -tw/2,-th/2,tw,th );
                    context.restore();


                    //  writes file stream
                    var fileStream = Yusef.mountpoints.storage.open( args.hash, opera.io.filemode.WRITE );
                    if( fileStream )
                    {
                        var dataUrl = canvas.toDataURL( 'image/jpeg', args.quality );
    
                        fileStream.writeBase64( dataUrl.slice( dataUrl[23]==','?24:23 ) );
                        fileStream.close();

                        //  update the cache
                        _updateCache( image, args.hash );
                    }

                    //  callback
                    callback
                    (
                        {
                            'file':     Yusef.mountpoints.storage.resolve( args.hash ),
                            'canvas':   canvas
                        }
                    );
                }
            
                //  GC
                canvas  = null;
                img     = null;
            }





            if( args.srcCanvas )
            {
                //  call resizeImage_handler on the args.srcCanvas
                resizeImage_handler
                (
                    {
                        'type':'load',
                        'srcElement':args.srcCanvas
                    }
                );
            }
            else
            {
                //  try to load the srcFile|image
                img         = new Image();
                img.onload  = image.onerror = resizeImage_handler;
                img.src     = unescape( (args.srcFile||image).path );
            }

            return false;
        };


        /*
         *  serveResizedImage
         */
        this.serveResizedImage = function( connection, image, args )
        {
            connection.pending = true;
            that.resizeImage
            (
                image,
                args,
                function( data )
                {
                    if( data.file && data.file.exists )
                    {
                        Yusef.serveFile( connection, data.file );
                    }
                    else
                    {
                        Yusef.emitErrorPage( connection );
                    }
                }
            );
        };
    }
);