/**
 * @fileOverview Plugin that adds UI information about the shared folder.
 *
 * In order to use this plugin in templates that support it you need to add {sharedFolderUI: true} into the <code>plugins</code> property of the object returned to the UI plugin.
 */
Yusef.addPlugin
(
    'sharedFolderUI',
    {
        Yusef_plugins_sharedFolderUI: function(id, path)
        {
            var self                = this;
            var ERR                 = 'Yusef.plugins.sharedFolderUI';
            var _content;
            var _selectSharedFolder;
            
            function init()
            {
                var publicHtml  = Yusef.mountpoints.application.resolve( path + '/public_html' );
                Yusef.shareStaticDirectory( publicHtml, id+'/' );
                
                var data = {
                    servicePath: opera.io.webserver.currentServicePath,
                    serviceName: opera.io.webserver.currentServiceName
                };
                var template = new Markuper(path + '/templates/sharedFolderUI.html', data);
                _content = template.parse().select();
                
                template = new Markuper(path + '/templates/selectSharedFolder.html', data);
                _selectSharedFolder = template.parse().select();
                
                extendUIPlugin();
            }

            function extendUIPlugin()
            {
                Yusef.plugins.ui.extend
                (
                    'formatData',
                    function extendedFormatData(data)
                    {
                        var userData = data.data;
                        
                        if( typeof(userData.plugins) == 'object'
                         && userData.plugins.sharedFolderUI == true )
                        {
                            userData.scripts = userData.scripts || [];
                            userData.scripts = userData.scripts.concat([{
                                src: 'libraries/dialog/dialog.js'
                            },{
                                src: 'libraries/dialog/ajax.js'
                            },{
                                src: 'libraries/dialog/json2.js'
                            }]);
                        }
                        
                        return extendedFormatData.proceed();
                    }
                );
            }

            this.getData = function(connection)
            {
                var data = {};

                data.js = 'static/'+id+'/sharedFolderUI.js';
                data.style = 'static/'+id+'/sharedFolderUI.css';
                data.content = _content;
                data.selectSharedFolderUI = _selectSharedFolder;
                
                return data;
            }

            init();
        }
    },
    function Yusef_plugins_sharedFolderUI_init()
    {
        this.Yusef_plugins_sharedFolderUI(this.yusefPluginId, this.path);
    }
)
