Yusef.addPlugin
(
    'translation',
    function Yusef_plugins_translation()
    {
        var self                = this;
        var config              = {domain: 'service'}; // TODO: should support different domains
        var translator          = new Translator( config );
        var original_           = translator.underscore;
        
        Yusef.addSectionListener( 'translation', translator.sendClientScript );
        
        function newRouteMethod(method)
        {
            return function(connection)
            {
                modifyTranslatorForTesting( connection, translator );
                translator.setLanguagesFromHttp( connection );
                var output = method.apply( this, arguments );
                if( !connection.closed 
                 && output && typeof(output.substring) == 'function' )
                {
                    return translator.translate(output);
                }
                else
                {
                    return output;
                }
            }
        }
        
        function extendAddAPIRoute(obj)
        {
            obj.extend
            (
                'addAPIRoute',
                function extendedAddAPIRoute(version, pattern, method, options)
                {
                    var newMethod = newRouteMethod(method);
                    return extendedAddAPIRoute.proceed(version, pattern, newMethod, options);
                }
            );
            
            return obj;
        }
        
        Yusef.extend
        (
            'addRoute',
            function extendedAddRoute( pattern, method, options )
            {
                if( (options||{}).translation !== false )
                {
                    var newMethod = newRouteMethod(method);
                }
                else
                {
                    var newMethod = method;
                }
                
                return extendedAddRoute.proceed(pattern, newMethod, options);
            }
        );

        extendAddAPIRoute(Yusef);
        // Unfortunately it wasn't possible to make _addAPIRoute a public method
        // due to security concerns. The only way then to extend this method
        // is using this hack (the problem is that the init function might
        // call the .addAPIRoute).
        Yusef.extend
        (
            'addPlugin',
            function extendedAddPlugin(id, obj, init)
            {
                var plugin = extendedAddPlugin.proceed(id, obj);
                
                extendAddAPIRoute(plugin);
                
                if( init )
                {
                    if( typeof(init)=='string' && init.match( /^[_a-z]\w*$/) && typeof(plugin[init])=='function' )
                    {
                        plugin[init]();
                        plugin[init] = null;
                        delete plugin[init];
                    }
                    else if(typeof(init)=='function' )
                    {
                        init.call( plugin );
                        init = null;
                    }
                }
            }
        );
        
        Yusef.extend
        (
            'emitErrorPage',
            function extendedEmitErrorPage( connection )
            {
                var output = extendedEmitErrorPage.proceed(connection);
                
                return translator.translate(output);
            }
        );

        if (window.markuper)
        {
            window.markuper.evaluator.registerFilter('getvar', function(input, expr) {
                var value = input;
                if( typeof input == 'string' )
                {
                    value = input.replace(/("|\\)/g, "\\$1");
                }
                return '"'+value+'" as ' + expr;
            });
        }
                
        window._ = translator.underscore;
        window.__ = translator.lazyUnderscore;
        
        this.translate = function(connection, str)
        {
            translator.setLanguagesFromHttp(connection);
            return translator.translate(str);
        };
        
        function modifyTranslatorForTesting( connection, translator )
        {
            if( !connection.request.queryItems.trans ) { return; }
            
            if( connection.request.queryItems.trans[0] == 'large' )
            {
                translator.underscore = function()
                {
                    var string = original_.apply( translator, arguments );
                    return string+string;
                }
                window._ = translator.underscore;
            }
            
            if( connection.request.queryItems.trans[0] == 'small' )
            {
                translator.underscore = function()
                {
                    var string = original_.apply( translator, arguments );
                    return string[0];
                }
                window._ = translator.underscore;
            }
        }
    }
);