/**
  * @fileOverview A Dialog library
  * 
  * 
  * Dialog.showMessage(message, options)
  * Dialog.showMessageYesNo(message, options)
  * Dialog.showMessageOkCancel(message, options)
  * Dialog.showBalloon(message, options)
  *
  * message: innerHTML or HTMLElement
  * options: {
  **  title: Title of the Dialog. "Alert", "Error", etc.
  **  margin: Number of pixels to usr around the dialog box
  **  closeButton: Whether to put a close button on the top right corner or not
  **  callback: The callback function, incase no buttons are provided
  *   
  *-  yes: callback
  *-  no: callback
  *
  *-  ok: callback
  *-  cancel: callback
  * }
  */
var Dialog = {
    defaultOptions:
    {
        buttons: [{ 
            label: 'OK', 
            callback: function(response){ 
                
            }
        }]
    },
    applyDefaults: function(destination, source)
    {
        if ( source == null || typeof(source) != 'object')
        {
            return destination || source;
        }
        destination = destination || ((source instanceof Array) ? [] : {});
        for ( var key in source )
        {
            destination[key] = this.applyDefaults(destination[key], source[key]);
        }
        return destination;
    },
    dismissDialog: function()
    {
        var panel = document.getElementById("dialogPanel");
        if(panel) panel.parentNode.removeChild(panel);
    },
    createDialog: function(message, options)
    {
        var self = this;
        var paddingLeftRight = '25px'; // space for list-style-type
        var paddingTopBottom = '7px';
        
        options = options || {};
        // FIXME: should reuse the dialog
        Dialog.dismissDialog();
        
        // IE my love...
        if( !window.innerWidth ) { window.innerWidth = document.documentElement.clientWidth; }
        if( !window.innerHeight ) { window.innerHeight = document.documentElement.clientHeight; }
        
        var panel = document.createElement('div');
        panel.id = 'dialogPanel';
        panel.style.backgroundColor = '#c1c1c1';
        //try
        //{
        //    // only for kick-ass browsers
        //    panel.style.backgroundColor = 'rgba(193,193,193,0.85)';
        //}
        //catch( ex )
        //{
        //    // there there
        //    panel.style.backgroundImage = 'url('+Unite.servicePath + '/static/libraries/dialog/overlay_bg.png)';
        //}
        
        panel.style.borderRadius = '5px 5px';
        panel.style.position = 'absolute';
        panel.style.zIndex = '9999';
        panel.style.padding = '35px 5px 0 0'
        
        // Title
        options.title = (options.title===undefined)?_("Alert"): options.title;
        if( options.title )
        {
            var title_panel = document.createElement('h3');
            
            title_panel.innerHTML = options.title;
            title_panel.style.textAlign = 'center';
            title_panel.style.paddingLeft = paddingLeftRight;
            title_panel.style.paddingRight = paddingLeftRight;
            title_panel.style.paddingTop = paddingTopBottom;
            panel.appendChild(title_panel);
        }
        
        // Close button
        if( options.closeButton )
        {
            var closeButton = document.createElement('button');
            closeButton.style.position = 'absolute';
            closeButton.style.right = '15px';//'5px';
            closeButton.style.top = '5px';
            closeButton.style.width = '26px';
            closeButton.style.height = '26px';
            closeButton.style.border = '0';
            closeButton.style.backgroundImage = 'url(' + Unite.servicePath + '/static/libraries/dialog/close.png)';
            closeButton.style.backgroundPosition = '0px 0px';
            closeButton.onmouseover = function() {
                closeButton.style.backgroundPosition = '0px -26px';
            }
            closeButton.onmouseout = function() {
                closeButton.style.backgroundPosition = '0px 0px';
            }
            closeButton.onclick = function() {
                // FIXME: doesn't belong here!
                Dialog.dismissBalloon();
            }
            panel.appendChild(closeButton);
        }
        
        // Message
        var message_pane = document.createElement('div');
        message_pane.style.maxWidth = Math.round(window.innerWidth*0.4)+'px';
        message_pane.style.maxHeight = Math.round(window.innerHeight*0.7)+'px';
        message_pane.style.overflow = 'auto';
        message_pane.style.paddingLeft = paddingLeftRight;
        message_pane.style.paddingRight = paddingLeftRight;
        if( !options.title ) {message_pane.style.paddingTop = paddingTopBottom;}
        // IE fails at ES-262 15.3.5.3 hence the typeof check
        if (typeof(message) == 'object' && message instanceof Element)
        {
            message_pane.appendChild(message);
        }
        else
        {
            message_pane.innerHTML = message;
        }
        panel.appendChild(message_pane);
        
        // Buttons
        var buttons_pane = document.createElement('div');
        buttons_pane.style.paddingLeft = paddingLeftRight;
        buttons_pane.style.paddingRight = paddingLeftRight;
        buttons_pane.style.paddingBottom = paddingTopBottom;
        buttons_pane.style.textAlign = 'center';
        
        for ( var i = 0; i < options.buttons.length; i++ ) { (function(i)
        {
            var button = options.buttons[i];
            var button_frame = document.createElement('button');
            button_frame.innerHTML = button.label;
            
            button_frame.onclick = function()
            {
                Dialog.dismissDialog();
                if ( button.callback ) button.callback(i, button.label);
                if(panel.parentNode) panel.parentNode.removeChild(panel);
            }
            buttons_pane.appendChild(button_frame);
        }).call(this, i)};
        panel.appendChild(buttons_pane);
        
        return panel;
    },
    // http://www.quirksmode.org/js/findpos.html
    findPos: function(obj)
    {
        var curleft = curtop = 0;
        if (obj.offsetParent) {
            do {
                curleft += obj.offsetLeft;
                curtop += obj.offsetTop;
            } while (obj = obj.offsetParent);
        }
        
        return {left: curleft, top: curtop}; 
    },
    dismissBalloon: function()
    {
        var panel = document.getElementById("dialogPanelArrow");
        if(panel) panel.parentNode.removeChild(panel);
        Dialog.dismissDialog();
    },
    // http://www.quirksmode.org/js/events_properties.html
    getMouseCoordinates: function(e) {
    	var posx = 0;
    	var posy = 0;
    	if (!e) var e = window.event;
    	if (e.pageX || e.pageY) 	{
    		posx = e.pageX;
    		posy = e.pageY;
    	}
    	else if (e.clientX || e.clientY) 	{
    		posx = e.clientX + document.body.scrollLeft
    			+ document.documentElement.scrollLeft;
    		posy = e.clientY + document.body.scrollTop
    			+ document.documentElement.scrollTop;
    	}
    	
    	return {x: posx, y: posy};
    },
    // Ugly function is ugly, should refactor and create some logic methods.
    // but there's no time for that for 10.5 :(
    showBalloon: function(message, options)
    {
        options = options || {};
        options.buttons = [];
        
        var dialog = Dialog.createDialog(message, options);
        var target = options && options.target;
        var margin = (options && options.margin) || 0;
        var img = document.createElement('div');
        var arrowSizeWidth = 35;
        var arrowSizeHeight = 18;
        
        Dialog.dismissBalloon();
        
        if( !target ) { return false; }
        
        dialog.style.visibility = 'hidden';
        document.body.appendChild(dialog);
        
        var width = dialog.offsetWidth + (margin*2) + arrowSizeHeight;
        var height = dialog.offsetHeight + (margin*2) + arrowSizeHeight;
        
        var dYabove; // delta_y_1; distance from the target's top border to the window top border
        var dYbelow; // delta_y_2; distance from the target's bottom border to the window bottom border
        var dXleft; // delta_x_1; distance from the target's left border to the window left border
        var dXright; // delta_x_1; distance from the target's right border to the window right border
        var pos = Dialog.findPos(target);
        
        dYabove = pos.top - window.pageYOffset;
        dYbelow = window.innerHeight - (dYabove+target.offsetHeight);
        dXleft = pos.left - window.pageXOffset;
        dXright = window.innerWidth - (dXleft+target.offsetWidth);
        // Calculate the best position
        
        // calculate the dimensions of each side
        var spaces = [
            {side: 'above', width: window.innerWidth, height: dYabove},
            {side: 'right', width: dXright, height: window.innerHeight},
            {side: 'below', width: window.innerWidth, height: dYbelow},
            {side: 'left', width: dXleft, height: window.innerHeight}
        ];
        //var deb = '';
        //alert('dYabove: ' + dYabove + '\ndYbelow: ' + dYbelow + '\ndXleft: ' + dXleft + '\ndXright: ' + dXright );
        
        // calculate the visible area of the balloon in each side (score)
        for( var i = 0; i < spaces.length; i++ )
        {
            spaces[i].area = spaces[i].width * spaces[i].height;
            spaces[i].score = (spaces[i].height>height?height:spaces[i].height)
                            * (spaces[i].width>width?width:spaces[i].width);
            spaces[i].fit   = spaces[i].area >= spaces[i].score;
            // introduce a shortcut
            spaces[spaces[i].side] = spaces[i];
            //deb += 'side: ' + spaces[i].side + '; area: ' + spaces[i].area + '; score: ' + spaces[i].score+'\n';
        }
        // alert(deb);
        
        // sort by space left available descending
        // and sub-order those that fit inside the space by space area.
        // this is because we want to display the balloon in the space
        // that has the biggest visible area when there's a fit.
        spaces.sort(function(a, b)
        {
            var scoreA = a.score;
            var scoreB = b.score;
            
            if( a.fit ) { scoreA += a.area; }
            if( b.fit ) { scoreB += b.area; }
            
            return scoreB - scoreA;
        });
        
        var selected = spaces[0];
        // If the selected area doesn't fit and it's either on the left or on 
        // the right then we use the space below the target since it's 
        // "easier" to expand the page downwards.
        // A similar problem exists when there isn't enough space at the top
        // of the page, in this case we also position the balloon below the
        // target.
        if( !selected.fit 
                && ((selected.side == 'left' || selected.side == 'right')
                 || (selected.side == 'above' && height > pos.top )) )
        {
            selected = spaces['below'];
        }
        
        var targetCenter = {
            left: pos.left + parseInt(target.offsetWidth/2),
            top: pos.top + parseInt(target.offsetHeight/2)
        };
        
        var top;
        var left;
        // position the dialog
        if( selected.side in {'above':1,'below':1} )
        {
            // align the center of the dialog with the center of the target
            left = (targetCenter.left-parseInt(width/2));
            var wallLeft = left - window.pageXOffset;
            var wallRight = (window.pageXOffset+window.innerWidth) - (left+width);
            // if the arrow is outside the dialog's border then move it inside
            if( selected.side == 'above' ) {
                top = pos.top - height
            } else {
                top = pos.top + target.offsetHeight + margin + arrowSizeHeight;
            }
            if( wallLeft < 0 ) { left -= wallLeft; }
            if( wallRight < 0 ) { left += wallRight; }
        }
        
        if( selected.side in {'left':1,'right':1} )
        {
            // align the center of the dialog with the center of the target
            top = (targetCenter.top-parseInt(height/2));
            var wallTop = top - window.pageYOffset;
            var wallBottom = (window.pageYOffset+window.innerHeight) - (top+height);
            if( selected.side == 'left' ) {
                left = pos.left - width;
            } else {
                left = pos.left + target.offsetWidth + margin + arrowSizeHeight;
            }
            
            // if the dialog gets outside the window then move it inside
            if( wallTop < 0 ) { top -= wallTop; }
            if( wallBottom < 0 ) { top += wallBottom; }
        }
        
        // position the arrow
        img.id = "dialogPanelArrow";
        img.style.backgroundImage = 'url('+Unite.servicePath + '/static/libraries/dialog/bg_overlay_arrows.png)';
        img.style.position = 'absolute';
        
        var arrowLeft;
        var arrowTop;
        var arrowWidth;
        var arrowHeight;
        var arrowBackgroundPosition;
        if( selected.side in {'above':1,'below':1} )
        {
            // align the center of the arrow with the center of the target
            arrowLeft = targetCenter.left - parseInt(arrowSizeWidth/2);
            if( arrowLeft < window.pageXOffset ) {
                arrowLeft = left+5;
            }
            if( arrowLeft+arrowSizeWidth > window.pageXOffset+window.innerWidth ) {
                arrowLeft = left+dialog.offsetWidth-arrowSizeWidth-5;
            }
            arrowWidth = arrowSizeWidth;
            arrowHeight = arrowSizeHeight;
            // if the arrow is outside the dialog's border then move it inside
            if( selected.side == 'above' ) {
                arrowTop = top+dialog.offsetHeight;
                arrowBackgroundPosition = '0px -35px';
            } else {
                arrowTop = top-arrowSizeHeight;
                arrowBackgroundPosition = '0px -53px';
            }
        }
        if( selected.side in {'left':1,'right':1} )
        {
            // align the center of the arrow with the center of the target
            arrowTop = targetCenter.top - parseInt(arrowSizeWidth/2);
            if( arrowTop < window.pageYOffset ) {
                arrowTop = top+5;
            }
            if( arrowTop+arrowSizeWidth > window.pageYOffset+window.innerHeight ) {
                arrowTop = top+dialog.offsetHeight-arrowSizeWidth-5;
            }
            arrowWidth = arrowSizeHeight;
            arrowHeight = arrowSizeWidth;
            // if the arrow is outside the dialog's border then move it inside
            if( selected.side == 'left' ) {
                arrowLeft = left+dialog.offsetWidth;
                arrowBackgroundPosition = '0px 0px';
            } else {
                arrowLeft = left-arrowSizeHeight;
                arrowBackgroundPosition = '-19px 0px';
            }
        }
        
        //img.style.border = '1px solid red'
        img.style.backgroundPosition = arrowBackgroundPosition;
        img.style.top = arrowTop+'px';
        img.style.left = arrowLeft+'px';
        img.style.width = arrowWidth+'px';
        img.style.height = arrowHeight+'px';
        img.style.zIndex = dialog.style.zIndex;
        document.body.appendChild(img);
        
        dialog.style.left = left+'px';
        dialog.style.top = top+'px';
        dialog.style.visibility = '';
        
        var oldOnClick = document.onclick;
        document.onclick = function onClick(event) {
            var coords = Dialog.getMouseCoordinates(event);
            if( coords.x < left || coords.x > left+width ||
                coords.y < top || coords.y > top+height ) {
                // FIXME: should use cross-browser event listeners
                Dialog.dismissBalloon();
                document.onclick = oldOnClick;
            }
            
            oldOnClick && oldOnClick.apply(this, arguments);
        }
    },
    showMessage: function(message, options)
    {
        var dialog = Dialog.createDialog(message, options);
        
        document.body.appendChild(dialog);
    },
    showMessageYesNo: function(message, options)
    {
        this.showMessage(message,
        {
            title: options.title,
            buttons: [
                {
                    label: 'Yes',
                    callback: options.yes
                },
                {
                    label: 'No',
                    callback: options.no
                }
            ]
        });
    },
    showMessageOkCancel: function(message, options)
    {
        this.showMessage(message,
        {
            title: options.title,
            buttons: [
                {
                    label: 'OK',
                    callback: options.ok
                },
                {
                    label: 'Cancel',
                    callback: options.cancel
                }
            ]
        });
    },
    showHelpMessage: function(id, options)
    {
        var ajax = new Opera.Net.Ajax();
        ajax.Get(Unite.servicePath + '/api/help/1.0/section/'+id, function(xhr)
        {
            if( !xhr.responseText ) { return false; }
            
            var xml;
            
            if( window.DOMParser )
            {
                var parser = new DOMParser();
                xml = parser.parseFromString(xhr.responseText,"text/xml");
            }
            else // Internet Explorer
            {
                xml = new ActiveXObject("Microsoft.XMLDOM");
                xml.async = "false";
                xml.loadXML(xhr.responseText);
            }
            
            var obj;
            var cdata;
            var result = xml.getElementsByTagName('result')[0];
            
            // Thanks IE for removing TextNodes \:D/
            if( result.childNodes.length > 1 ) {
                cdata = result.childNodes[1];
            } else if( result.childNodes.length == 1 ) {
                cdata = result.childNodes[0];
            }
            
            try
            {
                //obj = JSON.parse(cdata.nodeValue);
                obj = {html: cdata.nodeValue};
            }
            catch( ex )
            {
                return false;
            }
            
            options = options || {};
            options.title = '';
            options.closeButton = true;
            Dialog.showBalloon(obj.html, options);
        });
    }
}