Function.prototype.curry = function()
{
    var method  = this,
        args    = [].concat( arguments );
    return function()
    {
        return method.apply( this, args.concat( arguments ) );
    }
}
Function.prototype.bind = function( object )
{
    var method = this;
    return function ()
    {
        return method.apply( object, arguments );
    }
}

var ProxyClient = new function()
{
    var sid = '';
    if( document.cookie.match( /\bunite-session-id=([^;]+)/ ) )
    {
        sid = RegExp.$1;
    }
    var SM2_PROXYSERVER_URL = '/static/sm2proxyserver.html?sid=' + sid;
    var self = this;
    var _proxies = {};
    var _iframe = document.createElement( 'iframe' );
    _iframe.src = 'http://' + window.location.hostname + Unite.servicePath + SM2_PROXYSERVER_URL;
    //alert( _iframe.src );
    _iframe.style.display = 'none';
    
    function postMessage( message )
    {
        if( _iframe.contentWindow )
        {
//opera.postError( '[CLIENT] sending: ' + JSON.stringify( message ) );
            _iframe.contentWindow.postMessage( JSON.stringify( message ), '*' );
        }
        else
        {
//opera.postError( '[CLIENT] not iframe to send: ' + JSON.stringify( message ) );
        }
    }
    
    function processMessage( event )
    {
//opera.postError( '[CLIENT] receiving: ' + event.data );
        var message = JSON.parse( event.data );

        if( message.id && message.id in _proxies )
        {
            _proxies[message.id].__processMessage__( message );
        }
    }
    
    this.createProxy = function( id )
    {
        var proxy = {};
        
        proxy.__postMessage__ = function( message )
        {
            message.id = id;
            postMessage( message );
        }
        
        proxy.__dispatchMethodCall__ = function( methodName )
        {
            var message =
            {
                type        : 'methodCall',
                method      : methodName,
                arguments   : arguments.slice(1)
            }

            this.__postMessage__( message );
        }
        
        proxy.__dispatchWriteProperty__ = function( propertyName, value )
        {
            var message =
            {
                type        : 'write',
                property    : propertyName,
                value       : value
            }
            
            this.__postMessage__( message );
        }
        
        proxy.__processMessage__ = function( message )
        {
            switch( message.type )
            {
                case 'event':
                if( typeof this[message.name] === 'function' ) 
                {
                    this[message.name]();
                }
                
                case 'write':
                this[message.property] = message.value;
                break;
            }
        }
        
        _proxies[id] = proxy;
        return proxy;
    }
    
    this.init = function()
    {
        document.body.appendChild( _iframe );
        window.addEventListener( 'message', processMessage, false );
    }
}

function SoundManagerProxy()
{
    var self = this;
    var proxy = ProxyClient.createProxy( 'soundManager' );
    var stub = {};
    var _methods = [];
    var _events = ['onload'];
    var _properties = ['url'];
    
    _methods.map( function( method )
    {
        stub[method] = proxy.__dispatchMethodCall__.curry( method ).bind( proxy );
    });
    
    _events.map( function( event )
    {
        stub.__defineSetter__( event, function( fn )
        {
            proxy[event] = function() { fn.apply(stub); }
        });
    });
    
    _properties.map( function( property )
    {
        stub.__defineSetter__( property, function( value )
        {
            proxy.__dispatchWriteProperty__( property, value );
        });
    });
 
    // HACK HACK HACK
    stub.sounds = {};
    stub.soundIDs = [];
    stub.createSound = function()
    {
        var options = arguments[0];
        if( options.id in stub.sounds ) { return; }
        
        stub.soundIDs.push( options.id );
        stub.sounds[options.id] = new SMSoundProxy( options.id );
        proxy.__dispatchMethodCall__.apply( proxy, ['createSound'].concat(arguments) );
    }
    stub.destroySound = function( soundId )
    {
        var ix = stub.soundIDs.indexOf( soundId );
        stub.soundIDs.splice(ix, 1);
        delete stub.sounds[soundId];
        proxy.__dispatchMethodCall__.call( proxy, 'destroySound', soundId );
    }
    
    return stub;
}

function SMSoundProxy( id )
{
    var self = this;
    var proxy = ProxyClient.createProxy( id );
    var stub = {};
    var _methods = ['play', 'stop', 'unload', 'setPosition', 'togglePause'];
    var _properties = ['durationEstimate', 'position', 'bytesLoaded', 'bytesTotal'];
    
    _methods.map( function( method )
    {
        stub[method] = proxy.__dispatchMethodCall__.curry( method ).bind( proxy );
    });
    
    _properties.map( function( property )
    {
        stub.__defineSetter__( property, function( value )
        {
            proxy.__dispatchWriteProperty__( property, value );
        });
        
        stub.__defineGetter__( property, function()
        {
            // TODO: ask update?
            return proxy[property];
        });
    });
    
    stub.sID = id;
    stub.options = new SMSoundOptionsProxy( id + '__options__', stub );
    
    return stub;
}

function SMSoundOptionsProxy( id, soundStub )
{
    var self = this;
    var proxy = ProxyClient.createProxy( id );
    var stub = {};
    var _events = ['whileplaying', 'whileloading', 'onfinish'];
    
    _events.map( function( event )
    {
        stub.__defineSetter__( event, function( fn )
        {
            proxy[event] = function() { fn.apply(soundStub); }
        });
    });
    
    return stub;
}

//if( window.location.protocol == 'unite:' )
{
    window.soundManager = new SoundManagerProxy();
    window.addEventListener( 'load', function()
    {
        window.ProxyClient.init();
    }, false);
}