var mediaPlayer = new function()
{
    var ERR             = 'mediaPlayer.js';
    var ROOT_PATH       = 'mountpoint://shared/';
    var MEDIA_FILTER    = /(.+)\.(mp3|m4a|aac)$/i;
    var _activityStream = new ActivityStream();
    var webserver       = opera.io.webserver;
    
    function init()
    {
        Yusef.addSectionListener
        (
            '_index',
            function( connection )
            {
               connection.response.setStatusCode( '302', 'Found' );
               connection.response.setResponseHeader( 'Location', 'content/' );
               connection.response.close();
            }
        );
         
        Yusef.addSectionListener( 'content', handleContent );
        Yusef.addSectionListener( 'list', handleList );
        Yusef.addSectionListener( 'media', handleMedia, {ui: true} );
        Yusef.addSectionListener( 'search', handleSearch );
        Yusef.addSectionListener( 'playlist', handlePlaylist );
        Yusef.addSectionListener( 'image', handleImage );
        Yusef.addSectionListener( 'karaoke', handleKaraoke );
        Yusef.addSectionListener( 'null.mp3', handleNullMp3 );
        
        Yusef.addAPIRoute( '1.0', /list\/(.*)/, handleAPIList );
        
        Yusef.plugins.ui.setSkin( 'new' );
        Yusef.plugins.ui.setStyleSheet( "static/style.css" );
    }
    
    /****************
      Sections Logic
     ****************/
    function handleContent( connection )
    {
        var localData =
        {
            list        : getListElementFromPath( connection.request.path ),
            breadcrumbs : getBreadcrumbs( connection.request.path ),
            isOwner     : connection.isOwner
        }
        
        if( !localData.list ) { return Yusef.emitErrorPage( connection ); }
        
        localData.mediaFilesCount = localData.list.getElementsByClassName ? localData.list.getElementsByClassName('media').length : 0;
        
        return getYusefData( connection, localData );
    }
    
    function handleList( connection )
    {
        var node = getListElementFromPath( connection.request.path );
        if( !node ) { return Yusef.emitErrorPage( connection ); }
        return node.innerHTML || node;
    }
    
    // /search/<output>/<key1>/<value1>/.../<keyN>/<valueN>/
    // output: 'content', 'list', 'xml', 'json'
    function handleSearch( connection )
    {
        var response = connection.response;
        var outputs = ['content', 'list'];
        var root = Yusef.mountpoints.shared && Yusef.mountpoints.shared.resolve( ROOT_PATH );        
        var path = connection.request.path.slice(1).split('/');
        var output = path.shift();
        var args = {};
        var error;
        
        // the user must specify one of the possible outputs
        if( outputs.indexOf( output ) < 0 )
        {
            response.write( 'Please, specify one of the possible outputs: "' + outputs.join( '", "') + '"' );
            return response.close();
        }
        
        // create dictionary of search terms
        for( var i = 0; i < path.length; i += 2 )
        {
            args[path[i]] = (path[i+1]||'').toLowerCase();
        }
        
        // support for query items (javascript disabled on the client)
        args['name'] = args['name'] || getQueryItem( connection, 'name' );
        
        // check for errors
        if( !args['name'] || args['name'].length < 3 )
        {
            error = _('Please provide search text with at least 3 characters.');
        }
        
        // output the errors and get away from here
        if( error )
        {
            if( output == 'content' ) return {content: error};
            response.write( error );
            return response.close();
        }
        
        var startTime = new Date().getTime();
        var list = findFiles( root, args );
        var endTime = new Date().getTime();
        
        //opera.postError( 'Searching files: ' + (endTime-startTime)/1000 + 's: found ' + list.length + ' files.' );
        
        if( list.length == 0 )
        {
            error = _('No music found.');
        }
        
        switch( output )
        {
            case 'content':
            var localData = 
            {
                search  : args['name'],
                list    : error ? error : getListElement(list)
            }
            return getYusefData( connection, localData );
            break;

            case 'list':
            var node = error ? error : getListElement( getFileList(list) );
            response.write( node.innerHTML || node );
            return response.close();
            break;
        }
        
        return connection.response.close();
    }
    
    function handleMedia( connection )
    {
        var requestedFile = Yusef.mountpoints.shared && Yusef.mountpoints.shared.resolve( ROOT_PATH + connection.request.path );
        
        if( !requestedFile || !requestedFile.exists || requestedFile.isDirectory )
        {
            return Yusef.emitErrorPage( connection );
        }
        
        var mediaLink = getMediaLink(connection, connection.request.path, requestedFile);
        var fileName = unescape(requestedFile.name.substr(requestedFile.name.lastIndexOf("/")+1))
        _activityStream.publish("Playing " + requestedFile, ActivityVerbType.View, new ActivityObjectModel(ActivityObjectType.Audio, fileName, mediaLink), '<a href="' + mediaLink + '">Play</a>');
        
        Yusef.serveFile( connection, requestedFile );
    }
    
    function handlePlaylist( connection  )
    {
        var sliceIx = ('format' in connection.request.queryItems) ? 1 : 2;
        var path = connection.request.path.split('/').slice(sliceIx).join('/');
        var filename = opera.io.webserver.userName + '\'s media ' + unescape(path).replace(/\//g, ' > ' );
        var requestedFile = Yusef.mountpoints.shared && Yusef.mountpoints.shared.resolve( ROOT_PATH + decodeURIComponent(path) );
        var error;
        var playlist;
        var contentType;
        
        if( !requestedFile || !requestedFile.isDirectory )
        {
            return Yusef.emitErrorPage( connection );
        }
        else
        {
            requestedFile.refresh();
            var list = getFileList( requestedFile );
            var format = (connection.request.path.split( '/' )[1]||'').toLowerCase()
                      || getQueryItem( connection, "format" );
            
            playlist = getPlaylist( connection, format, list );
            if( playlist === false )
            {
                error = _("<STRONG>{format}</STRONG> is an unsupported format.");
            }
            else
            {
                showPlaylistNotification(connection, "/"+unescape(path));
            }
        }
        
        if( !error )
        {
            var contentType = getContentTypeFor( format )
            connection.response.setResponseHeader( 'Content-Type', contentType  );
            connection.response.setResponseHeader( 'Content-Disposition' , 'attachment; filename=' + filename + '.' + format );
        }
        
        connection.response.write( playlist || error );
        connection.response.close();
    }
    
    function handleImage( connection )
    {
        var path = connection.request.path;
        var file = Yusef.mountpoints.shared.resolve(path);
        var nocover = Yusef.mountpoints.application.resolve('public_html/img/default-cover.png');
        var image;
        
        if( file.exists )
        {
            ID3.loadTags(file.path, function()
            {
                image = ID3.getTag(file.path, 'image');
            }, ["APIC", "PIC"]);
            
            if( image )
            {
                connection.response.setResponseHeader( 'Content-Type', image.format );
                connection.response.writeBytes( image.data );
                connection.response.close();
            }
            else
            {
                // try getting the image from the file <music>.jpg
                ['png', 'jpg', 'jpeg', 'gif'].forEach(function(ext)
                {
                    if ( !(image && image.exists) )
                    {
                        image = Yusef.mountpoints.shared.resolve
                        (
                             file.path.replace(/(.+\.)[^\.]+$/, '$1'+ext)
                        );
                    }
                });
                
                image = image.exists ? image : nocover;
                // FIXME: it's not always png
                connection.response.setResponseHeader( 'Content-Type', 'image/png' );
                var stream = image.open(image, 'r');
                if( image === nocover )
                {
                    connection.response.setStatusCode('404', 'Not Found');
                }
                connection.response.writeBytes( stream.readBytes(stream.bytesAvailable) );
                connection.response.close();
                stream.close();
            }
        }

    }
    
    function handleKaraoke( connection )
    {
        var path = connection.request.path;
        var file = Yusef.mountpoints.shared.resolve(path.replace(/\.[^\.]+$/, '.txt'));
        var json;
        
        if( file.exists )
        {
            var stream = file.open(file, 'r');
            var ultrastar = new Ultrastar( stream.read(stream.bytesAvailable) );
            stream.close();
            json = ultrastar.getLyrics();
        }
        else
        {
            json = {error: file.name + ' not found.'};
        }
        
        connection.response.write( JSON.stringify(json) );
        connection.response.close();
    }
    
    function handleNullMp3( connection )
    {
        var file = Yusef.mountpoints.application.resolve( 'media/null.mp3' );
        return Yusef.serveFile( connection, file );
    }
    
    /***************
      API Functions
     ***************/
    function handleAPIList( connection, matches )
    {
        var path = matches[0];
        var result = '';
        var format = connection.request.queryItems.json ? 'json' : (connection.request.queryItems.xml ? 'xml' : 'html');
        
        switch( format )
        {
            case 'html':
                var node = getListElementFromPath( path );
                if( node ) { result = node.innerHTML || node; }
                break;
                
            case 'json':
                var list = getFileListFromPath( path );
                if( list ) { result = JSON.stringify( list ); }
                break;
                
            case 'xml':
                var list = getFileListFromPath( path );
                if( list )
                {
                    var tmpl = new Markuper( '/templates/list.xml' );
                    tmpl.parse({media: list});
                    result = tmpl.xml();
                }
                break;
        }
        
        connection.response.write( result );
        connection.response.close();
    }
    
    /******************
      Helper Functions
     ******************/
    function showPlaylistNotification( connection, path )
    {
        if( connection.isOwner ) { return; }
        
        widget.showNotification(_("A visitor downloaded a playlist of {folderName}.", {folderName: path}));
    }
     
    function getPlaylist( connection, format, list )
    {
        switch( format )
        {
            case 'm3u'  : return getM3UPlaylist( connection, list ); break;
        }
        
        return false;
    }
    
    /*
     * http://schworak.com/programming/music/playlist_m3u.asp
     */
    function getM3UPlaylist( connection, list )
    {
        var m3u = "#EXTM3U\n";
        if(Yusef.plugins.acl)
        {
            aclData = Yusef.plugins.acl.getData( connection );
            magicPrefix = "?_magic=" + aclData.acl.magic + "&session-redirect=noCookie";
        }
        for( var i = 0, item; item = list[i]; i++ )
        {
            if( item.type == 'folder' ) { continue; }
            var link;
            if( !Yusef.plugins.directConnection || !( link = Yusef.plugins.directConnection.getLink(item.location) ))
            {
                if(aclData && aclData.acl.type == "limited")
                {
                    //link = "http://" + opera.io.webserver.hostName + opera.io.webserver.currentServicePath + magicPrefix + item.location;
                    link = "http://" + opera.io.webserver.hostName + opera.io.webserver.currentServicePath + item.location + magicPrefix;
                } else {
                    link = "http://" + opera.io.webserver.hostName + opera.io.webserver.currentServicePath + item.location;
                }
            } else {
                if(aclData && aclData.acl.type == "limited")
                {
                    link = Yusef.plugins.directConnection.getLink( item.location + magicPrefix);
                }
            }
            m3u += "#EXTINF:," + item.name + "\n";
            m3u += link + "\n";
        }
        
        return m3u;
    }
     
    function getContentTypeFor( format )
    {
        switch( format )
        {
            case 'm3u': return 'audio/x-mpegurl';
        }
    }
     
    function getMediaLink( connection, path, requestedFile )
    {
        var fullPath        = opera.io.webserver.currentServicePath;
        var IdForFile       = generateIdForFile( requestedFile)
        var splitPath       = IdForFile.split("/");
        var folder          = splitPath.splice(0, splitPath.length-1).join("/");
        var fullPathAbsolute = 'http://'+connection.request.headers.Host[0] + fullPath + "content/#list=list/" + folder + "&mid=" + IdForFile;
        return fullPathAbsolute;
    }
    
    function getFileFromPath( path )
    {
        var requestedFile = Yusef.mountpoints.shared && Yusef.mountpoints.shared.resolve( ROOT_PATH + path );
//opera.postError( 'URI: ' + connection.request.uri );
//opera.postError( 'path: ' + ROOT_PATH + path );
//opera.postError( 'path: ' + requestedFile.path );
//opera.postError( 'exists: ' + requestedFile.exists );
        if( !requestedFile || !requestedFile.isDirectory )
        {
            return false;
        }
        
        requestedFile.refresh();
        return requestedFile;
    }
    
    function getFileListFromPath( path )
    {
        var requestedFile = getFileFromPath( path );
        
        var startTime = new Date().getTime();
        var fileList = requestedFile && getFileList( requestedFile );
        var endTime = new Date().getTime();
        //PSO.pub( ERR,  'getFileList: ' + (endTime-startTime)/1000 + 's' );
        
        return fileList;
    }
    
    function getListElementFromPath( path )
    {
        var requestedFile = getFileFromPath( path );
        var fileList = getFileListFromPath( path );
        
        if( ! fileList ) { return false; }
                
        if( fileList.length==0 )
        {
            fileList.unshift
            ({
                id      : generateIdForFile( requestedFile.parent ),
                name    : _('No music found.'),
                number  : "",
                nameKey : '..',
                type    : 'noMusicFound'
            });
        }
        
        if( path != '/' && requestedFile.parent )
        {
            fileList.unshift
            ({
                id      : generateIdForFile( requestedFile.parent ),
                name    : _('Parent folder'),
                number  : "",
                nameKey : '..',
                type    : 'folder parent',
                location: 'content/' + requestedFile.parent.path.slice( ROOT_PATH.length ) + '/'
            });
        }
        
        var node = getListElement( fileList );
        
        return node;
    }

    function getYusefData( connection, localData )
    {
        localData = localData || {};
        
        var startTime = new Date().getTime();
        var data = Yusef.getData( connection );
        var endTime = new Date().getTime();
        //PSO.pub( ERR,  'Yusef.getData: ' + (endTime-startTime)/1000 + 's' );
        
        data.plugins = Yusef.getPluginsData( connection );
        
        var tmpl = new Markuper( "templates/mediaPlayer.html", data );
        var side = new Markuper( 'templates/sidebar.html', data );
        
        localData.search = localData.search || '';
        localData.list = localData.list || '';
        localData.breadcrumbs = localData.breadcrumbs || [];
        localData.breadcrumb = localData.breadcrumbs.splice(-1, 1)[0];
        localData.parentDirectory = (localData.breadcrumbs.splice(-1, 1)||[])[0];
        
        var startTime = new Date().getTime();
        tmpl.parse(localData);
        side.parse(localData);
        var endTime = new Date().getTime();
        PSO.pub( ERR,  'mediaPlayer.html: ' + (endTime-startTime)/1000 + 's' );
        
        var tabs = [
            {"title": opera.io.webserver.currentServiceName, "section": "content", "url": opera.io.webserver.currentServicePath , "class": 'active'},
            {"title": _("Activity Feed"), "section": "activity"}
        ];
        
        if( connection.isOwner )
        {
            tabs.push({"title": _("Help"), "section": "help"});
        }
        
        return {
            content : tmpl.select(),
            tabs    : Yusef.plugins.ui.generateTabs(tabs, connection),
            sidebar : side.select(),
            //styles  : [{href: 'style.css'}],
            scripts : [//{src: 'soundmanager2-nodebug-jsmin.js'},
                       {src: 'soundmanager2.js'}, // used for debug
                       {src: 'flash_detect.js'},
                       {src: 'ajax.js'},
                       {src: 'json2.js'},
                       {src: 'main.js'}],
            plugins : {
                sharedFolderUI  : true
            }
        };
    }
    
    function getListElement( listData )
    {
        var data = {servicePath: opera.io.webserver.currentServicePath.slice(0, -1)};
        var tmpl = new Markuper( 'templates/list.html', data );
        
        return tmpl.parse({media: listData}).select('#songs')[0];
    }
    
    function findFiles( dir, args )
    {
        var list = [];
        //opera.postError( 'searching in ' + dir.path );
        dir.refresh();
        for( var i = 0, file; file = dir[i]; i++ )
        {
            if( file.isDirectory )
            {
                list = list.concat( findFiles( file, args ) );
            }
            else if( file.name.toLowerCase().indexOf( args['name'] ) >= 0 )
            {
                list.push( file );
            }
        }
        
        return list;
    }
    
    // converts text into a string that can be usable in id or name attributes
    // for referecing in fragments.
    // ID and NAME tokens must begin with a letter ([A-Za-z]) and may be
    // followed by any number of letters, digits ([0-9]), hyphens ("-"),
    // underscores ("_"), colons (":"), and periods (".").
    // Note: / seems safe however
    function getSafeIdHash( text )
    {
        //return escape(text).replace( /%/g, ':' );
        //return encodeURIComponent(text)
        //return text
        return encodeURIComponent(unescape(text))
               .replace( /%2F/ig, "/" ) // don't encode "/"
               .replace( /'/g, escape("'") )
               .replace( /:/g, escape(":") )
               .replace( /%/g, ':' );
    }
    
    function getWebPath( path )
    {
        //return encodeURIComponent(unescape(path)).replace(/'/g, escape("'"));
        return path.replace(/'/g, escape("'"));
    }
    
    function generateIdForFile( file )
    {
        //var path = unescape(file.path.slice( ROOT_PATH.length ));
        var path = file.path.slice( ROOT_PATH.length );
        
        return generateIdForPath( path );
    }
    
    function generateIdForPath( path )
    {
        return getSafeIdHash( path );
    }
    
    function getFileList( files )
    {
        var list = [];
        var startTime = new Date().getTime();
        
        //var TAGS = [];
        //for( var key in ID3v2.frames )
        //{
        //    TAGS.push(key);
        //}
        
        for( var i=0, file; file=files[i]; i++ )
        {
            if( !file.name.match(MEDIA_FILTER) && !file.isDirectory ) { continue; };
            var type = RegExp.$2.toLowerCase(); // mp3|m4a
            var name = unescape( file.isDirectory ? file.name : RegExp.$1 );
            var tags = {};
            
            if( !file.isDirectory )
            {
                ID3.loadTags(file.path, function()
                {
                    tags = ID3.getAllTags(file.path);
                }/*, TAGS*/);
                //PSO.pub( 'afonso', 'tags: ' + JSON.stringify(tags, undefined, '  ') );
            }
            
            // FIXME: no track 0 allowed...
            var track = Number((tags.track||"").split('/')[0]);
            track = track == 0 ? "" : track;
            
            list.push
            ({
                id          : generateIdForFile( file ),
                name        : tags.title || name,
                number      : track,
                nameKey     : tags.title || name,
                artistKey   : tags.artist || '',
                albumKey    : tags.album || '',
                numberKey   : track,
                type        : file.isDirectory ? 'folder' : 'media ' + type,
                location    : (file.isDirectory ? 'content/' : 'media/') + getWebPath(file.path.slice( ROOT_PATH.length )) + (file.isDirectory?'/':'')
                //id3         : JSON.stringify(tags, undefined, '  ')
                // encodeURIComponent(decodeURI(file.path.slice( ROOT_PATH.length ))) );
            });
        }
        
        var endTime = new Date().getTime();
        //opera.postError( 'getFileList: ' + (endTime-startTime)/1000 + 's' );
        
        return list;
    }
    
    function getBreadcrumbs( path )
    {
        path = opera.io.webserver.currentServiceName + path.slice(0, -1);
        
        var link = '';
        var breadcrumbs = [];
        var crumbs = unescape( path ).split('/');
        //opera.postError( 'crumbs: ' + crumbs );
        for( var i = 0, crumb; crumb = crumbs[i]; i++ )
        {
            link += (i == 0 ? 'content' : crumb) + '/';
            breadcrumbs.push
            ({
                crumb   : crumb,
                link    : link,
                id      : generateIdForPath( link.replace('content/', '') ),
            });
        }
        
        return breadcrumbs;
    }
    
    function getQueryItem( connection, name )
    {
        return (connection.request.queryItems[name]||[])[0];
    }
    
    init();
}